package basic

import (
	"math"

	"b612.me/astro/planet"
	. "b612.me/astro/tools"
)

func JupiterL(JD float64) float64 {
	return planet.WherePlanet(4, 0, JD)
}

func JupiterB(JD float64) float64 {
	return planet.WherePlanet(4, 1, JD)
}
func JupiterR(JD float64) float64 {
	return planet.WherePlanet(4, 2, JD)
}
func AJupiterX(JD float64) float64 {
	l := JupiterL(JD)
	b := JupiterB(JD)
	r := JupiterR(JD)
	el := planet.WherePlanet(-1, 0, JD)
	eb := planet.WherePlanet(-1, 1, JD)
	er := planet.WherePlanet(-1, 2, JD)
	x := r*Cos(b)*Cos(l) - er*Cos(eb)*Cos(el)
	return x
}

func AJupiterY(JD float64) float64 {

	l := JupiterL(JD)
	b := JupiterB(JD)
	r := JupiterR(JD)
	el := planet.WherePlanet(-1, 0, JD)
	eb := planet.WherePlanet(-1, 1, JD)
	er := planet.WherePlanet(-1, 2, JD)
	y := r*Cos(b)*Sin(l) - er*Cos(eb)*Sin(el)
	return y
}
func AJupiterZ(JD float64) float64 {
	//l := JupiterL(JD)
	b := JupiterB(JD)
	r := JupiterR(JD)
	//	el := planet.WherePlanet(-1, 0, JD)
	eb := planet.WherePlanet(-1, 1, JD)
	er := planet.WherePlanet(-1, 2, JD)
	z := r*Sin(b) - er*Sin(eb)
	return z
}

func AJupiterXYZ(JD float64) (float64, float64, float64) {
	l := JupiterL(JD)
	b := JupiterB(JD)
	r := JupiterR(JD)
	el := planet.WherePlanet(-1, 0, JD)
	eb := planet.WherePlanet(-1, 1, JD)
	er := planet.WherePlanet(-1, 2, JD)
	x := r*Cos(b)*Cos(l) - er*Cos(eb)*Cos(el)
	y := r*Cos(b)*Sin(l) - er*Cos(eb)*Sin(el)
	z := r*Sin(b) - er*Sin(eb)
	return x, y, z
}

func JupiterSeeRa(JD float64) float64 {
	lo, bo := JupiterSeeLoBo(JD)
	sita := Sita(JD)
	ra := math.Atan2((Sin(lo)*Cos(sita) - Tan(bo)*Sin(sita)), Cos(lo))
	ra = ra * 180 / math.Pi
	return Limit360(ra)
}
func JupiterSeeDec(JD float64) float64 {
	lo, bo := JupiterSeeLoBo(JD)
	sita := Sita(JD)
	dec := ArcSin(Sin(bo)*Cos(sita) + Cos(bo)*Sin(sita)*Sin(lo))
	return dec
}

func JupiterSeeRaDec(JD float64) (float64, float64) {
	lo, bo := JupiterSeeLoBo(JD)
	sita := Sita(JD)
	ra := math.Atan2((Sin(lo)*Cos(sita) - Tan(bo)*Sin(sita)), Cos(lo))
	ra = ra * 180 / math.Pi
	dec := ArcSin(Sin(bo)*Cos(sita) + Cos(bo)*Sin(sita)*Sin(lo))
	return Limit360(ra), dec
}

func EarthJupiterAway(JD float64) float64 {
	x, y, z := AJupiterXYZ(JD)
	to := math.Sqrt(x*x + y*y + z*z)
	return to
}

func JupiterSeeLo(JD float64) float64 {
	x, y, z := AJupiterXYZ(JD)
	to := 0.0057755183 * math.Sqrt(x*x+y*y+z*z)
	x, y, z = AJupiterXYZ(JD - to)
	lo := math.Atan2(y, x)
	bo := math.Atan2(z, math.Sqrt(x*x+y*y))
	lo = lo * 180 / math.Pi
	bo = bo * 180 / math.Pi
	lo = Limit360(lo)
	//lo-=GXCLo(lo,bo,JD)/3600;
	//bo+=GXCBo(lo,bo,JD);
	lo += HJZD(JD)
	return lo
}

func JupiterSeeBo(JD float64) float64 {
	x, y, z := AJupiterXYZ(JD)
	to := 0.0057755183 * math.Sqrt(x*x+y*y+z*z)
	x, y, z = AJupiterXYZ(JD - to)
	//lo := math.Atan2(y, x)
	bo := math.Atan2(z, math.Sqrt(x*x+y*y))
	//lo = lo * 180 / math.Pi
	bo = bo * 180 / math.Pi
	//lo+=GXCLo(lo,bo,JD);
	//bo+=GXCBo(lo,bo,JD)/3600;
	//lo+=HJZD(JD);
	return bo
}

func JupiterSeeLoBo(JD float64) (float64, float64) {
	x, y, z := AJupiterXYZ(JD)
	to := 0.0057755183 * math.Sqrt(x*x+y*y+z*z)
	x, y, z = AJupiterXYZ(JD - to)
	lo := math.Atan2(y, x)
	bo := math.Atan2(z, math.Sqrt(x*x+y*y))
	lo = lo * 180 / math.Pi
	bo = bo * 180 / math.Pi
	lo = Limit360(lo)
	//lo-=GXCLo(lo,bo,JD)/3600;
	//bo+=GXCBo(lo,bo,JD);
	lo += HJZD(JD)
	return lo, bo
}

func JupiterMag(JD float64) float64 {
	AwaySun := JupiterR(JD)
	AwayEarth := EarthJupiterAway(JD)
	Away := planet.WherePlanet(-1, 2, JD)
	i := (AwaySun*AwaySun + AwayEarth*AwayEarth - Away*Away) / (2 * AwaySun * AwayEarth)
	i = ArcCos(i)
	Mag := -9.40 + 5*math.Log10(AwaySun*AwayEarth) + 0.0005*i
	return FloatRound(Mag, 2)
}
